#pragma once
#ifndef Camera_h
#define Camera_h

#include <vector>

#include <Eigen/Core>
#include <Eigen/Geometry>

class Camera
{
public:
	/// Creates a view projection matrix for the provided camera properties
	const Eigen::Matrix4f& update(
		const Eigen::Vector2f& viewPortSize,
		const Eigen::Vector4f& position,
		const Eigen::Vector4f& forward,
		const Eigen::Vector4f& up,
		float verticalFovDeg,
		float aspect,
		float zNear,
		float zFar);

private:
	/// Creates a view matrix to transform world space vertices into the camera space defined by the provided parameters
	Eigen::Matrix4f createViewMatrix();
	/// Creates a perspective projection matrix to transform view space vertices into the clip space
	Eigen::Matrix4f createPerspectiveProjection();

public:
	Eigen::Vector4f mPosition = Eigen::Vector4f::UnitW();
	Eigen::Vector4f mRight = Eigen::Vector4f::UnitX();
	Eigen::Vector4f mForward = Eigen::Vector4f::UnitY();
	Eigen::Vector4f mUp = Eigen::Vector4f::UnitZ();
	Eigen::Matrix4f mViewMatrix = Eigen::Matrix4f::Identity();
	Eigen::Matrix4f mInverseViewMatrix = Eigen::Matrix4f::Identity();
	Eigen::Matrix4f mProjectionMatrix = Eigen::Matrix4f::Identity();
	Eigen::Matrix4f mViewProjectionMatrix = Eigen::Matrix4f::Identity();
	Eigen::Vector2f mViewPortSize = Eigen::Vector2f::Zero();
	float mVerticalFieldOfView = 0.0f; ///< Vertical field of view in radians
	float mHorizontalFieldOfView = 0.0f; ///< Horizontal field of view in radians
	float mAspectRatio = 0.0f;
	float mZNear = 0.0f;
	float mZFar = 0.0f;
};

#endif