#pragma once
#ifndef Heightfield_h
#define Heightfield_h

#include <memory>

#include <Eigen/Core>
#include <GL/glew.h>

#include "Camera.h"
#include "Program.h"

namespace Eigen
{
	typedef Matrix<unsigned int, 2, 1> Vector2ui;
}

struct ChunkDescriptor
{
	const int mDesiredNumChunks = 2048;
	int mChunkDivisor;
	Eigen::Vector2i mChunkGridDimensions;
	Eigen::Vector2f mChunkCellSize;
	int mNumChunks;
};

//=================================================================================================
/**
*	This class describes the domain on which the heightfield is defined.
*
*	@author	Daniel Cornel
**/
//=================================================================================================
class Heightfield
{
public:
	/// Constructor
	Heightfield(const Eigen::Vector2i& dimensions, const Eigen::Vector2f& cellSize, const Eigen::Vector2f& origin, float scaleFactor);
	/// Destructor
	~Heightfield();

public:
	Eigen::Vector2i mDimensions = Eigen::Vector2i::Ones();	///< The number of cells in either dimension
	Eigen::Vector2f mCellSize = Eigen::Vector2i::Ones();	///< The extents of a cell in world units
	Eigen::Vector2f mOrigin = Eigen::Vector2f::Zero();		///< The offset of the bottom left corner in world units
	Eigen::AlignedBox3f mAABB = Eigen::AlignedBox3f();		///< The axis-aligned bounding box of the entire heightfield
	float mScaleFactor = 1.0f;								///< The scale factor of the noise function, indicates the maximum value of the heightfield

	GLuint mBuffer = 0u;	///< Id of the heightfield value buffer
	std::unique_ptr<Program> mNoiseProgram = nullptr;	///< Compute program to create the heightfield value buffer from a noise function

	ChunkDescriptor mChunkDescriptor = ChunkDescriptor();	///< Descriptor of the chunk level used for tessellation
};

#endif