#pragma once
#ifndef TessellationKerblEtAl2022_h
#define TessellationKerblEtAl2022_h

#include "../AbstractTessellation.h"
#include "../Program.h"

//=================================================================================================
/**
*	This class implements a compute-shader-based tessellation by Bernhard Kerbl, Linus Horvath,
*	Daniel Cornel, and Michael Wimmer. An Improved Triangle Encoding Scheme for Cached
*	Tessellation. Eurographics 2022 - Short Papers, pages 53-56, 2022.
*
*	@author	Daniel Cornel
**/
//=================================================================================================
class TessellationKerblEtAl2022 : public AbstractTessellation
{
public:
	/// Constructor
	TessellationKerblEtAl2022(const Heightfield& heightfield);
	/// Destructor
	~TessellationKerblEtAl2022() override;
	/// Performs a per-frame update of the tessellation
	void update(const Camera& camera) override;
	/// Renders the geometry
	void render(const Camera& camera) override;

private:
	/// Performs one subdivision/merge iteration
	void subdivideMerge(const Camera& camera);
	/// Resets the temporary buffers and creates the command for the next update dispatch
	void resetBuffers();

public:
	GLuint mTriangleInstanceVertexBuffer = 0u;
	GLuint mTriangleInstanceIndexBuffer = 0u;
	GLuint mGridVertexBuffer = 0u;
	GLuint mGridIndexBuffer = 0u;
	GLuint mSubdivisionBuffer1 = 0u;
	GLuint mSubdivisionBuffer2 = 0u;
	GLuint mCulledTrianglesBuffer = 0u;
	GLuint mNumInputTrianglesBuffer = 0u;
	GLuint mNumOutputTrianglesBuffer = 0u;
	GLuint mIndirectDispatchBuffer = 0u;
	GLuint mIndirectDrawBuffer = 0u;
	GLuint mChangesAtomicCounterBuffer = 0u;
	GLuint mVAO = 0u;
	bool mPingPong = false;

	std::unique_ptr<Program> mTerrainProgram = nullptr;
	std::unique_ptr<Program> mLoDProgram = nullptr;
	std::unique_ptr<Program> mIndirectProgram = nullptr;
};

#endif