#pragma once
#ifndef TessellationLeeEtAl2013_h
#define TessellationLeeEtAl2013_h

#include "../AbstractTessellation.h"
#include "../Program.h"

//=================================================================================================
/**
*	This class implements recursive tessellation as described by Hyunjin Lee, Yuna Jeong, and
*	Sungkil Lee. Recursive tessellation. In ACM SIGGRAPH Asia Posters, page 16:1. ACM, 2013.
*
*	@author	Daniel Cornel
**/
//=================================================================================================
class TessellationLeeEtAl2013 : public AbstractTessellation
{
public:
	/// Constructor
	TessellationLeeEtAl2013(const Heightfield& heightfield);
	/// Destructor
	~TessellationLeeEtAl2013() override;
	/// Performs a per-frame update of the tessellation
	void update(const Camera& camera) override;
	/// Renders the geometry
	void render(const Camera& camera) override;

public:
	GLuint mChunkVertexBuffer = 0u;	///< OpenGL handle of the chunk vertex buffer
	GLuint mVertexBuffer0 = 0u;		///< OpenGL handle of the first ping-pong vertex buffer
	GLuint mVertexBuffer1 = 0u;		///< OpenGL handle of the second ping-pong vertex buffer
	GLuint mAtomicCounter = 0u;		///< OpenGL handle of the atomic counter buffer
	GLuint mVAO = 0u;				///< OpenGL handle of the vertex array object

	std::unique_ptr<Program> mProgram = nullptr;	///< The tessellation program used for rendering
};

#endif