#pragma once
#ifndef TessellationNaive_h
#define TessellationNaive_h

#include "../AbstractTessellation.h"
#include "../Program.h"

//=================================================================================================
/**
*	This class provides a naive tessellation by triangulating all heightfield cells without LoD as
*	a baseline method.
*
*	@author	Daniel Cornel
**/
//=================================================================================================
class TessellationNaive : public AbstractTessellation
{
public:
	/// Constructor
	TessellationNaive(const Heightfield& heightfield);
	/// Destructor
	~TessellationNaive() override;
	/// Performs a per-frame update of the tessellation
	void update(const Camera& camera) override;
	/// Renders the geometry
	void render(const Camera& camera) override;

public:
	GLuint mVertexBuffer = 0u;	///< OpenGL handle of the vertex buffer
	GLuint mIndexBuffer = 0u;	///< OpenGL handle of the vertex index buffer
	GLuint mVAO = 0u;			///< OpenGL handle of the vertex array object

	std::unique_ptr<Program> mProgram = nullptr;	///< The program used for rendering the naive heightfield tessellation
};

#endif