#pragma once
#ifndef PipelineStatisticsQuery_h
#define PipelineStatisticsQuery_h

#include <unordered_map>
#include <vector>

#include <GL/glew.h>

namespace PipelineStatistics
{
	static const std::vector<GLenum> sPipelineStatistics = {
		GL_PRIMITIVES_GENERATED,
		GL_VERTICES_SUBMITTED_ARB,
		GL_PRIMITIVES_SUBMITTED_ARB,
		GL_VERTEX_SHADER_INVOCATIONS_ARB,
		GL_TESS_CONTROL_SHADER_PATCHES_ARB,
		GL_TESS_EVALUATION_SHADER_INVOCATIONS_ARB,
		GL_GEOMETRY_SHADER_INVOCATIONS,
		GL_GEOMETRY_SHADER_PRIMITIVES_EMITTED_ARB,
		GL_CLIPPING_INPUT_PRIMITIVES_ARB,
		GL_CLIPPING_OUTPUT_PRIMITIVES_ARB,
		GL_FRAGMENT_SHADER_INVOCATIONS_ARB,
		GL_SAMPLES_PASSED,
		GL_COMPUTE_SHADER_INVOCATIONS_ARB,
		GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN,
		GL_TIME_ELAPSED,
	};

	static const std::unordered_map<GLenum, std::string> sPipelineStatisticsStrings = {
		{ GL_PRIMITIVES_GENERATED, "Primitives generated" },
		{ GL_VERTICES_SUBMITTED_ARB, "Vertices submitted" },
		{ GL_PRIMITIVES_SUBMITTED_ARB, "Primitives submitted" },
		{ GL_VERTEX_SHADER_INVOCATIONS_ARB, "Vertex shader invocations" },
		{ GL_TESS_CONTROL_SHADER_PATCHES_ARB, "Tessellation control shader patches" },
		{ GL_TESS_EVALUATION_SHADER_INVOCATIONS_ARB, "Tessellation evaluation shader invocations" },
		{ GL_GEOMETRY_SHADER_INVOCATIONS, "Geometry shader invocations" },
		{ GL_GEOMETRY_SHADER_PRIMITIVES_EMITTED_ARB, "Geometry shader primitives emitted" },
		{ GL_CLIPPING_INPUT_PRIMITIVES_ARB, "Clipping input primitives" },
		{ GL_CLIPPING_OUTPUT_PRIMITIVES_ARB, "Clipping output primitives" },
		{ GL_FRAGMENT_SHADER_INVOCATIONS_ARB, "Fragment shader invocations" },
		{ GL_SAMPLES_PASSED, "Samples passed" },
		{ GL_COMPUTE_SHADER_INVOCATIONS_ARB, "Compute shader invocations" },
		{ GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN, "Transform feedback primitives written" },
		{ GL_TIME_ELAPSED, "Time elapsed" },
	};
};

//=================================================================================================
/**
*	This class provides a static debug query for OpenGL pipeline statistics. To use it, simply
*	surround one or multiple OpenGL draw calls with the start()/stop() functions and then dump or
*	inspect the results.
*
*	@author	Daniel Cornel
**/
//=================================================================================================
class PipelineStatisticsQuery
{
public:
	/// Creates OpenGL query objects for the different statistics and starts the recording
	static void start(const std::vector<GLenum>& statistics);
	static void start(GLenum statistic);
	static void start();
	/// Stops the recording and destroys the OpenGL query objects for the different statistics
	static bool stop();

	/// Format the time given in nanoseconds into a string with appropriate units
	static std::string formatTimeString(GLuint timeInNS);
private:
	/// Writes all query results to stdout
	static void dump(bool error);

private:
	static std::vector<std::pair<GLenum, GLuint>> sQueryObjects;
	static std::vector<std::pair<std::string, GLuint>> sQueryResults;
};

#endif